"""
This modules is a wrapper around nbconvert
It provides a cleaner version of the HTML content that can be embedded into
existing HTML pages without breaking existing styles.
"""

import io
import json
import logging
import os

import jupytext
import mistune
from nbconvert.exporters import HTMLExporter, MarkdownExporter
from nbconvert.filters.highlight import _pygments_highlight
from nbconvert.filters.markdown_mistune import (
    IPythonRenderer,
    MarkdownWithMath,
)
from nbconvert.nbconvertapp import NbConvertApp
from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import get_lexer_by_name
from pygments.util import ClassNotFound

from mkdocs_jupyter.config import settings
from mkdocs_jupyter.preprocessors import SubCell

logger = logging.getLogger("mkdocs.mkdocs-jupyter")

# Language of the kernel (used for syntax highlighting)
kernel_lang = "python"

# We use this to tag each div with code with an unique ID for the copy-to-clipboard
cell_id = 0


def nb2html(
    nb_path, execute=False, kernel_name="", theme=None, start=0, end=None
):
    """
    Convert a notebook to HTML

    Arguments
    ---------
        nb_path: str
            Path to the notebook
        execute: bool
            Whether to execute the notebook
        kernel_name: str
            Name of the kernel to use
        theme: str
            Name of the theme to use (default: light) (options: light or dark)
        start: int
            Start cell number
        end: int
            End cell number

    Returns
    -------
        HTML content
    """
    logger.info(f"Converting notebook (execute={execute}): {nb_path}")

    global cell_id, kernel_lang
    cell_id = 0  # Reset the cell id

    app = get_nbconvert_app(
        execute=execute, kernel_name=kernel_name, start=start, end=end
    )

    # Use the templates included in this package
    template_file = "mkdocs_html/notebook.html.j2"
    extra_template_paths = [settings.templates_dir]

    # Customize NBConvert App
    preprocessors_ = [SubCell]
    filters = {
        "highlight_code": custom_highlight_code,
        "markdown2html": custom_markdown2html,
    }

    exporter = HTMLExporter(
        config=app.config,
        template_file=template_file,
        extra_template_paths=extra_template_paths,
        preprocessors=preprocessors_,
        filters=filters,
        theme=theme,
    )

    _, extension = os.path.splitext(nb_path)

    if extension == ".py":
        nb = jupytext.read(nb_path)
        nb_file = io.StringIO(jupytext.writes(nb, fmt="ipynb"))
        content, resources = exporter.from_file(nb_file)
    else:
        try:
            with open(nb_path, "r", encoding="utf-8") as f:
                nb_json = json.load(f)
                kernel_lang = nb_json["metadata"]["kernelspec"]["language"]
        except KeyError:
            pass
        content, resources = exporter.from_filename(nb_path)

    return content


def nb2md(nb_path, start=0, end=None, execute=False, kernel_name=""):
    """Convert a notebook to markdown

    We use a template that removed all code cells because if the body
    is to big ( with javascript and CSS) it takes to long to read and parse
    """

    app = get_nbconvert_app(
        start=start, end=end, execute=execute, kernel_name=kernel_name
    )

    # Use the templates included in this package
    template_file = "mkdocs_md/md-no-codecell.md.j2"
    extra_template_paths = [settings.templates_dir]

    exporter = MarkdownExporter(
        config=app.config,
        template_file=template_file,
        extra_template_paths=extra_template_paths,
    )

    _, extension = os.path.splitext(nb_path)

    if extension == ".py":
        nb = jupytext.read(nb_path)
        nb_file = io.StringIO(jupytext.writes(nb, fmt="ipynb"))
        body, resources = exporter.from_file(nb_file)
    else:
        body, resources = exporter.from_filename(nb_path)
    return body


def get_nbconvert_app(
    execute=False, kernel_name="", start=0, end=None
) -> NbConvertApp:
    """Create"""

    # Load the user's nbconvert configuration
    app = NbConvertApp()
    app.load_config_file()

    app.config.update(
        {
            # This Preprocessor changes the pygments css prefixes
            # from .highlight to .highlight-ipynb
            "CSSHTMLHeaderPreprocessor": {
                "enabled": True,
                "highlight_class": ".highlight-ipynb",
            },
            "SubCell": {"enabled": True, "start": start, "end": end},
            "ExecutePreprocessor": {
                "enabled": execute,
                "store_widget_state": True,
                "kernel_name": kernel_name,
                "allow_errors": True,
            },
        }
    )

    return app


def custom_highlight_code(source, language=None, metadata=None):
    """
    Change CSS class names from .highlight to .highlight-ipynb.
    This are for the <div> that contains the <pre>

    This modifies only the HTML not CSS.
    On the `notebook.html.js` template we modify the CSS styles.
    """
    global cell_id
    cell_id = cell_id + 1
    if not language:
        language = kernel_lang

    formatter = HtmlFormatter(cssclass="highlight-ipynb hl-" + language)
    output = _pygments_highlight(source, formatter, language, metadata)

    clipboard_copy_txt = f"""<div id="cell-{cell_id}" class="clipboard-copy-txt">{source}</div>
    """
    return output + clipboard_copy_txt


def custom_markdown2html(source):
    """
    This filter uses a Custom Rendered that uses a custom CodeHtmlFormatter
    to wrap the language blocks into a <div> and a <pre> tags.
    This is done so it's the same HTML structure that for regular non-language
    sections.
    """
    return MarkdownWithMath(
        renderer=CustomMarkdownRendered(escape=False)
    ).render(source)


class CustomMarkdownRendered(IPythonRenderer):
    def block_code(self, code, lang):
        lexer = None
        if lang:
            try:
                lexer = get_lexer_by_name(lang, stripall=True)
            except ClassNotFound:
                code = lang + "\n" + code
                lang = None

        if not lang:
            return "\n<pre><code>%s</code></pre>\n" % mistune.escape(code)

        formatter = CodeHtmlFormatter()
        return highlight(code, lexer, formatter)


class CodeHtmlFormatter(HtmlFormatter):
    def wrap(self, source, outfile):
        return self._wrap_code(source)

    def _wrap_code(self, source):
        yield 0, '<div class="codehilite"><pre><code>'
        for i, t in source:
            yield i, t
        yield 0, "</code></pre></div>"


if __name__ == "__main__":
    content = nb2html("tests/mkdocs/docs/demo.ipynb")
    with open("./demo.html", "w") as f:
        f.write(content)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mkdocs_jupyter',
 'mkdocs_jupyter.tests',
 'mkdocs_jupyter.tests.mkdocs.docs',
 'mkdocs_jupyter.tests.mkdocs.docs.extras']

package_data = \
{'': ['*'],
 'mkdocs_jupyter': ['templates/mkdocs_html/*',
                    'templates/mkdocs_html/assets/*',
                    'templates/mkdocs_md/*'],
 'mkdocs_jupyter.tests': ['mkdocs/*', 'mkdocs/overrides/*'],
 'mkdocs_jupyter.tests.mkdocs.docs': ['img/*']}

install_requires = \
['jupytext>=1.11.02,<2.0.0',
 'mkdocs-material>=8.0.0,<9.0.0',
 'mkdocs>=1.2.3,<2.0.0',
 'nbconvert>=6.2.0,<7.0.0']

entry_points = \
{'mkdocs.plugins': ['mkdocs-jupyter = mkdocs_jupyter.plugin:Plugin']}

setup_kwargs = {
    'name': 'mkdocs-jupyter',
    'version': '0.20.1',
    'description': 'Use Jupyter in mkdocs websites',
    'long_description': '<p align="center">\n    <img src="https://raw.githubusercontent.com/danielfrg/mkdocs-jupyter/main/docs/logo.png" width="450px">\n</p>\n\n<p align="center">\n    <a href="https://pypi.org/project/mkdocs-jupyter/">\n        <img src="https://badge.fury.io/py/mkdocs-jupyter.svg">\n    </a>\n    <a href="https://github.com/danielfrg/mkdocs-jupyter/actions/workflows/test.yml">\n        <img src="https://github.com/danielfrg/mkdocs-jupyter/workflows/test/badge.svg">\n    </a>\n    <a href="https://codecov.io/gh/danielfrg/mkdocs-jupyter?branch=main">\n        <img src="https://codecov.io/gh/danielfrg/mkdocs-jupyter/branch/main/graph/badge.svg">\n    </a>\n    <a href="http://github.com/danielfrg/mkdocs-jupyter/blob/main/LICENSE.txt">\n        <img src="https://img.shields.io/:license-Apache%202-blue.svg">\n    </a>\n</p>\n\n\n# mkdocs-jupyter: Use Jupyter Notebooks in mkdocs\n\n- Add Jupyter Notebooks directly to the mkdocs navigation\n- Support for multiple formats:\n  - `.ipynb` and `.py` files (using [jupytext](https://github.com/mwouts/jupytext))\n- Same style as regular Jupyter Notebooks\n  - Support Jupyter Themes\n- Option to execute the notebook before converting\n- Support for [ipywidgets](https://github.com/jupyter-widgets/ipywidgets)\n- Support for mkdocs TOC\n- Option to include notebook source\n\n<a href="https://raw.githubusercontent.com/danielfrg/mkdocs-jupyter/master/docs/mkdocs-theme.png"><img src="https://raw.githubusercontent.com/danielfrg/mkdocs-jupyter/master/docs/mkdocs-theme.png" alt="mkdocs-jupyter default theme"  width="410"></a>\n<a href="https://raw.githubusercontent.com/danielfrg/mkdocs-jupyter/master/docs/material-theme.png"><img src="https://raw.githubusercontent.com/danielfrg/mkdocs-jupyter/master/docs/material-theme.png" alt="mkdocs-jupyter material theme"  width="410"></a>\n\n## Installation\n\n```shell\npip install mkdocs-jupyter\n```\n\n## Configuration\n\nIn the `mkdocs.yml` use Jupyter notebooks (`.ipynb`) or Python scripts (`.py`) as pages:\n\n```yaml\nnav:\n- Home: index.md\n- Notebook page: notebook.ipynb\n- Python file: python_script.py\n\nplugins:\n  - mkdocs-jupyter\n```\n\n### Titles and Table of Contents\n\nThe first h1 header (`#`) in your notebook will be used as the title.\n\n```md\n# This H1 header will be the the title.\n```\n\nThis can be turned off in the configuration (in which case the filename will be used as title):\n\n```yaml\nplugins:\n  - mkdocs-jupyter:\n      ignore_h1_titles: True\n```\n\nIn order to see the table of contents you need to maintain a hierarchical headers structure in your notebooks.\nYou must use h2 headers (`##`) and not h1 (`#`)\n\n```md\n## This H2 title will show in the table of contents\n```\n\nIf you want to **nest headers** in the TOC you need to add additional levels later\nin the same markdown cell or new bottom markdown cells:\n\n```md\n## This header will show as top level in the table of contents\n\n<content>\n\n### This one will be displayed inside the above level\n```\n\n### Including or Ignoring Files\n\nYou can control which files are included or ignored via lists of glob patterns:\n\n```yaml\nplugins:\n  - mkdocs-jupyter:\n      include: ["*.ipynb"]  # Default: ["*.py", "*.ipynb"]\n      ignore: ["some-irrelevant-files/*.ipynb"]\n```\n\n### Execute Notebook\n\nYou can tell the plugin to execute the notebook before converting, default is `False`:\n\n```yaml\nplugins:\n  - mkdocs-jupyter:\n      execute: True\n```\n\nYou can tell the plugin to ignore the execution of some files (with glob matching):\n\n```yaml\nplugins:\n  - mkdocs-jupyter:\n      execute_ignore: "my-secret-files/*.ipynb"\n```\n\n#### Kernel\n\nBy default the plugin will use the kernel specified in the notebook to execute it.\nYou can specify a custom kernel name to use for all the notebooks:\n\n```yaml\nplugins:\n  - mkdocs-jupyter:\n      kernel_name: python3\n```\n\n### Jupyter themes\n\nYou can configure the different Jupyter themes.\nFor example if using material with `slate` color scheme you can use the Jupyter Lab `dark` theme:\n\n```yml\nplugins:\n  - mkdocs-jupyter:\n      theme: dark\n\ntheme:\n  name: material\n  palette:\n    scheme: slate\n```\n\n### Download notebook link\n\nYou can tell the plugin to include the notebook source to make it easy to show\na download button in the theme, default is `False`:\n\n```yml\nplugins:\n  - mkdocs-jupyter:\n      include_source: True\n```\n\nThis setting will also create a `page.nb_url` value that you can use in your theme\nto make a link in each page.\n\nFor example in `mkdocs-material`\n(see [customization](https://squidfunk.github.io/mkdocs-material/customization/#overriding-template-blocks)),\nyou can create a `main.html` file like this:\n\n```jinja\n{% extends "base.html" %}\n\n{% block content %}\n{% if page.nb_url %}\n    <a href="{{ page.nb_url }}" title="Download Notebook" class="md-content__button md-icon">\n        {% include ".icons/material/download.svg" %}\n    </a>\n{% endif %}\n\n{{ super() }}\n{% endblock content %}\n```\n\n![Download Notebook button](https://raw.githubusercontent.com/danielfrg/mkdocs-jupyter/master/docs/download-button.png)\n\n### Styles\n\nThis extensions includes the Jupyter Lab nbconvert CSS styles and does some changes\nto make it as generic as possible in order for it to work with a variety of mkdocs themes.\nThis is not always possible and the theme we test the most is [mkdocs-material](https://squidfunk.github.io/mkdocs-material).\n\nIt\'s possible you might need to do some CSS changes to make it look as good as you\nwant, for example for the material theme take a look at their [customization docs](https://squidfunk.github.io/mkdocs-material/customization/#overriding-template-blocks).\n\nCreate a `main.html` file like:\n\n```jinja\n{% extends "base.html" %}\n\n{% block content %}\n{{ super() }}\n\n<style>\n// Do whatever changes you need here\n\n.jp-RenderedHTMLCommon p {\n    color: red\n}\n\n</style>\n{% endblock content %}\n```\n',
    'author': 'Daniel Rodriguez',
    'author_email': 'None',
    'maintainer': 'Daniel Rodriguez',
    'maintainer_email': 'None',
    'url': 'https://github.com/danielfrg/mkdocs-jupyter',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<4',
}


setup(**setup_kwargs)
